local QJN_Defaults = { };
QJN_Defaults["CHAT_ENABLED"] = false;
QJN_Defaults["TOASTS_ENABLED"] = true;
QJN_Defaults["TOASTFRAME_WIDTH"] = 350;
QJN_Defaults["TOASTFRAME_HEIGHT"] = 80;
QJN_Defaults["TOASTFRAME_PADDING"] = 10;
QJN_Defaults["TOASTFRAME_MARGIN"] = 15;
QJN_Defaults["TOASTCONTAINER_YOFFSET"] = 40;
QJN_Defaults["TOASTCONTAINER_MAXTOASTS"] = 3;
QJN_Defaults["TOASTFRAME_GROWDIRECTION"] = "TOP";

local function loadOptions()
	if ( type(QJN_Options) ~= "table" ) then QJN_Options = { } end;
	for k,v in pairs(QJN_Defaults) do
		if ( type(QJN_Options[k]) ~= type(v) ) then
			QJN_Options[k] = v;
		end
	end
	
	if QJN_Options["TOASTFRAME_GROWDIRECTION"] == "TOP" then
		QJN_Options["TOASTFRAME_ANCHORPOS"] = "BOTTOM";
	else
		QJN_Options["TOASTFRAME_ANCHORPOS"] = "TOP";
	end
end

local toastBG = {
  -- path to the background texture
  bgFile = "Interface\\FriendsFrame\\UI-Toast-Background",  
  -- path to the border texture
  edgeFile = "Interface\\FriendsFrame\\UI-Toast-Border",
  -- true to repeat the background texture to fill the frame, false to scale it
  tile = true,
  -- size (width or height) of the square repeating background tiles (in pixels)
  tileSize = 12,
  -- thickness of edge segments and square size of edge corners (in pixels)
  edgeSize = 12,
  -- distance from the edges of the frame to those of the background texture (in pixels)
  insets = {
	left = 5,
	right = 5,
	top = 5,
	bottom = 5
  }
}

local toastContainer;

local function createToastFrame(friendName, queueFlavorText, queueDescription, id, parent)
	local newToastFrame = CreateFrame("FRAME", "quickJoinToastFrame"..id, parent or UIParent);
	newToastFrame:SetPoint(QJN_Options["TOASTFRAME_GROWDIRECTION"]);
	newToastFrame:SetSize(QJN_Options["TOASTFRAME_WIDTH"], QJN_Options["TOASTFRAME_HEIGHT"]);
	newToastFrame:SetBackdrop(toastBG);
	newToastFrame.id = id;
	
	newToastFrame:SetScript("OnMouseUp", function (self, button)
		if ( button == "LeftButton" ) then
			--from FriendsFrame.lua - ToggleFriendsSubPanel
			--PanelTemplates_SetTab(FriendsFrame, 1); --Friends panel
			--PanelTemplates_SetTab(FriendsTabHeader, 2); --QuickJoin subpanel
			--FriendsFrame_Update();
			--ShowUIPanel(FriendsFrame);
			ToggleQuickJoinPanel() --maybe use this to close panel on second click?
		elseif ( button == "RightButton" ) then
			self:close();
		end
	end);
	
	newToastFrame.friendName = newToastFrame:CreateFontString(newToastFrame:GetName() .. "FriendName", "OVERLAY", "FriendsFont_Normal");
	local fontName, fontSize, flags = newToastFrame.friendName:GetFont();
	newToastFrame.friendName:SetFont(fontName, fontSize+2, flags);
	newToastFrame.friendName:SetText(friendName);
	newToastFrame.friendName:SetPoint("TOPLEFT", QJN_Options["TOASTFRAME_PADDING"], -1 * QJN_Options["TOASTFRAME_PADDING"]);
	
	newToastFrame.isQueuedText = newToastFrame:CreateFontString(newToastFrame:GetName() .. "IsQueuedText", "OVERLAY", "FriendsFont_Normal");
	newToastFrame.isQueuedText:SetTextColor(0.8, 0.8, 0.8, 1);
	newToastFrame.isQueuedText:SetWidth(QJN_Options["TOASTFRAME_WIDTH"] - 3 * QJN_Options["TOASTFRAME_PADDING"] - newToastFrame.friendName:GetWidth());
	newToastFrame.isQueuedText:SetWordWrap(false);
	newToastFrame.isQueuedText:SetJustifyH("LEFT");
	newToastFrame.isQueuedText:SetText(queueFlavorText);
	newToastFrame.isQueuedText:SetPoint("LEFT", newToastFrame.friendName, "RIGHT", 0, 0);
	
	newToastFrame.queueName = newToastFrame:CreateFontString(newToastFrame:GetName() .. "QueueName", "OVERLAY", "FriendsFont_Normal");
	newToastFrame.queueName:SetTextColor(0.8, 0.8, 0.8, 1);
	newToastFrame.queueName:SetWidth(QJN_Options["TOASTFRAME_WIDTH"] - 3 * QJN_Options["TOASTFRAME_PADDING"]);
	newToastFrame.queueName:SetHeight(QJN_Options["TOASTFRAME_HEIGHT"] - 4 * QJN_Options["TOASTFRAME_PADDING"] - newToastFrame.friendName:GetHeight());
	newToastFrame.queueName:SetWordWrap(true);
	newToastFrame.queueName:SetJustifyH("LEFT");
	newToastFrame.queueName:SetJustifyV("BOTTOM");
	newToastFrame.queueName:SetText(queueDescription);
	newToastFrame.queueName:SetPoint("BOTTOMLEFT", QJN_Options["TOASTFRAME_PADDING"], QJN_Options["TOASTFRAME_PADDING"]);
	
	newToastFrame.animation = newToastFrame:CreateAnimationGroup();
	newToastFrame.animation.fadeInAnimation = newToastFrame.animation:CreateAnimation("Alpha");
	newToastFrame.animation.fadeInAnimation:SetDuration(0.2);
	newToastFrame.animation.fadeInAnimation:SetSmoothing("IN");
	newToastFrame.animation.fadeInAnimation:SetFromAlpha(0);
	newToastFrame.animation.fadeInAnimation:SetToAlpha(1);
	newToastFrame.animation.fadeInAnimation:SetOrder(1);
	newToastFrame.animation.fadeOutAnimation = newToastFrame.animation:CreateAnimation("Alpha");
	newToastFrame.animation.fadeOutAnimation:SetStartDelay(4.05);
	newToastFrame.animation.fadeOutAnimation:SetDuration(1.5);
	newToastFrame.animation.fadeOutAnimation:SetSmoothing("OUT");
	newToastFrame.animation.fadeOutAnimation:SetFromAlpha(1);
	newToastFrame.animation.fadeOutAnimation:SetToAlpha(0);
	newToastFrame.animation.fadeOutAnimation:SetOrder(2);
	newToastFrame.animation.fadeOutAnimation:SetScript("OnFinished", function(self)
		self:GetParent():GetParent():close();
	end);
	
	newToastFrame.setFriendName = function (self, friendName)
		self.friendName:SetText(friendName);
		self.isQueuedText:ClearAllPoints();
		self.isQueuedText:SetPoint("LEFT", self.friendName, "RIGHT", 0, 0);
	end
	newToastFrame.setIsQueuedText = function (self, isQueuedText)
		self.isQueuedText:SetText(isQueuedText);
	end
	newToastFrame.setQueueName = function (self, queueName)
		self.queueName:SetText(queueName);
	end
	newToastFrame.close = function(self)
		toastContainer:close(self.id);
	end
	newToastFrame.refreshFontStringSize = function(self)
		self.isQueuedText:SetWidth(QJN_Options["TOASTFRAME_WIDTH"] - 3 * QJN_Options["TOASTFRAME_PADDING"] - self.friendName:GetWidth());
		self.queueName:SetWidth(QJN_Options["TOASTFRAME_WIDTH"] - 3 * QJN_Options["TOASTFRAME_PADDING"]);
		self.queueName:SetWidth(QJN_Options["TOASTFRAME_WIDTH"] - 3 * QJN_Options["TOASTFRAME_PADDING"]);
	end
	
	return newToastFrame;
end

local function init_toastContainer()
	toastContainer = CreateFrame("FRAME", "quickJoinToastContainer", UIParent);
	toastContainer:SetSize(QJN_Options["TOASTFRAME_WIDTH"], QJN_Options["TOASTCONTAINER_MAXTOASTS"] * (QJN_Options["TOASTFRAME_HEIGHT"] + QJN_Options["TOASTFRAME_MARGIN"]));
	toastContainer:SetPoint(QJN_Options["TOASTFRAME_ANCHORPOS"] .. DEFAULT_CHAT_FRAME.buttonSide, DEFAULT_CHAT_FRAME, QJN_Options["TOASTFRAME_GROWDIRECTION"] .. DEFAULT_CHAT_FRAME.buttonSide, 0, QJN_Options["TOASTCONTAINER_YOFFSET"]);
	
	toastContainer.toastQueue = {};
	toastContainer.nextToastID = QJN_Options["TOASTCONTAINER_MAXTOASTS"] + 1;
	
	--hide a frame, update index, reposition all frames and look if there are new toasts to update in toastQueue
	toastContainer.close = function (self, closeID)
		local lastRepositionedToastFrame = nil;
		local closedToastFrame = nil;
		
		for k, toast in pairs(self.toast) do
			if ( toast.id == closeID ) then
				toast:Hide();
				toast.id = self.nextToastID;
				self.nextToastID = self.nextToastID + 1;
				closedToastFrame = toast;
			end
		end

		table.sort(self.toast, function (toast1, toast2)
			return toast1.id < toast2.id;
		end);
		for k, toast in pairs(self.toast) do
			toast:ClearAllPoints();
			if ( lastRepositionedToastFrame == nil ) then
				toast:SetPoint(QJN_Options["TOASTFRAME_ANCHORPOS"] .. DEFAULT_CHAT_FRAME.buttonSide, toastContainer, QJN_Options["TOASTFRAME_ANCHORPOS"] .. DEFAULT_CHAT_FRAME.buttonSide);
			else
				toast:SetPoint(QJN_Options["TOASTFRAME_ANCHORPOS"], lastRepositionedToastFrame, QJN_Options["TOASTFRAME_GROWDIRECTION"], 0, QJN_Options["TOASTFRAME_MARGIN"]);
			end;
			lastRepositionedToastFrame = toast;
		end;
		
		self:processQueue();
	end;
	
	toastContainer.processQueue = function (self)
		if ( #self.toastQueue == 0 ) then
			return; --no new toasts to display
		end;
	
		local availableToastIndex = -1;
		local lowestToastID = self.nextToastID + 1; --higher number than all existing IDs
		for index, toast in pairs(self.toast) do
			if ( not toast:IsShown() and toast.id < lowestToastID ) then
				availableToastIndex = index;
				lowestToastID = toast.id;
			end;
		end;
		if ( availableToastIndex == -1 ) then
			return; --no toastFrame is currently available to receive information
		end;
		
		local toastData = table.remove(self.toastQueue, 1);
		self.toast[availableToastIndex]:setFriendName(toastData["friendName"]);
		self.toast[availableToastIndex]:setIsQueuedText(toastData["queueFlavorText"]);
		self.toast[availableToastIndex]:setQueueName(toastData["queueDescription"]);
		self.toast[availableToastIndex]:refreshFontStringSize();
		self.toast[availableToastIndex]:Show();
		self.toast[availableToastIndex].animation:Stop();
		self.toast[availableToastIndex].animation:Play();
		PlaySoundKitID(79739); --UI_71_Social_Queueing_Toast
		
		--process the queue again if there was more than one data waiting and more than one frame available
		self:processQueue();
	end;
	
	toastContainer.addToast = function(self, friendName, queueFlavorText, queueDescription)
		local toastData = { ["friendName"] = friendName, ["queueFlavorText"] = queueFlavorText, ["queueDescription"] = queueDescription };
		table.insert(self.toastQueue, toastData);
		self:processQueue();
	end
	
	toastContainer.refreshFrameSize = function(self) 
		for i=1,QJN_Options["TOASTCONTAINER_MAXTOASTS"] do
			toastContainer.toast[i]:SetSize(QJN_Options["TOASTFRAME_WIDTH"], QJN_Options["TOASTFRAME_HEIGHT"]);
			toastContainer.toast[i]:refreshFontStringSize();
		end
	end
	
	--create and reuse frames;
	toastContainer.toast = {};
	for i=1,QJN_Options["TOASTCONTAINER_MAXTOASTS"] do
		toastContainer.toast[i] = createToastFrame("", "", "", i, toastContainer);
		toastContainer.toast[i]:Hide();
		
		if ( i == 1 ) then
			toastContainer.toast[i]:ClearAllPoints();
			toastContainer.toast[i]:SetPoint(QJN_Options["TOASTFRAME_ANCHORPOS"] .. DEFAULT_CHAT_FRAME.buttonSide, toastContainer, QJN_Options["TOASTFRAME_ANCHORPOS"] .. DEFAULT_CHAT_FRAME.buttonSide);
		else
			toastContainer.toast[i]:ClearAllPoints();
			toastContainer.toast[i]:SetPoint(QJN_Options["TOASTFRAME_ANCHORPOS"], toastContainer.toast[i-1], QJN_Options["TOASTFRAME_GROWDIRECTION"], 0, QJN_Options["TOASTFRAME_MARGIN"]);
		end;
	end;
end;

function QJN_addQuickJoinNotification(friendName, queueFlavorText, queueDescription)
	if (QJN_Options["CHAT_ENABLED"]) then
		DEFAULT_CHAT_FRAME:AddMessage("|cff00ff00QuickJoin: |r"..friendName..queueFlavorText.." "..queueDescription)
	end
	if (QJN_Options["TOASTS_ENABLED"]) then
		toastContainer:addToast(friendName, queueFlavorText, queueDescription)
	end
end

function QJN_applyOptions()
	toastContainer:refreshFrameSize()
end

--Event frame
local QJNframe = CreateFrame("Frame", "QJN_Eventframe");
QJNframe:RegisterEvent("SOCIAL_QUEUE_UPDATE");
QJNframe:RegisterEvent("PLAYER_LOGIN");

function QJNframe:OnEvent(event, ...)
	if ( event == "SOCIAL_QUEUE_UPDATE" ) then
		local guid, numAddedItems = ...;
		if ( numAddedItems == 0 or C_SocialQueue.GetGroupMembers(guid) == nil) then
			--this seems to indicate that no additional data is provided for this event, therefore we ignore it
			return;
		end
		
		local members, playerName, color;
		if C_SocialQueue.GetGroupMembers(guid) ~= nil then
			members = SocialQueueUtil_SortGroupMembers(C_SocialQueue.GetGroupMembers(guid));
			playerName, color = SocialQueueUtil_GetNameAndColor(members[1]);
		end
		
		local queues = C_SocialQueue.GetGroupQueues(guid);
		if queues ~= nil then
			if ( queues[1].type == "lfglist" ) then
				if ( queues[1].eligible ) then
					--lfglist groups can't queue in the regular LFG tool, so we only need to check this queue
					local queue = queues[1];
					
					local id, activityID, name, comment, voiceChat, iLvl, honorLevel, age, numBNetFriends, numCharFriends, numGuildMates, isDelisted, leaderName, numMembers = C_LFGList.GetSearchResultInfo(queue.lfgListID);
					local activityName, shortName, categoryID, groupID, minItemLevel, filters, minLevel, maxPlayers, displayType, _, useHonorLevel = C_LFGList.GetActivityInfo(activityID);
					--local shortComment = string.sub(comment, 0, 30).."...";
					if ( comment == "" ) then
						QJN_addQuickJoinNotification(playerName, " is looking for members:", activityName..": "..name);
					else
						QJN_addQuickJoinNotification(playerName, " is looking for members:", activityName..": "..name.."\n("..comment..")");
					end
				end
			else
				--maybe several queues, concat all of them for displaying
				local queueSummaryName = "";
				local anyEligibleQueue = false;
				for id, queue in pairs(queues) do
					if ( queue.eligible ) then
						anyEligibleQueue = true;
						local queueName = SocialQueueUtil_GetQueueName(queue);
						if ( queueSummaryName == "" ) then
							queueSummaryName = queueName;
						else
							queueSummaryName = queueSummaryName..", "..queueName;
						end
					end
				end
				if ( anyEligibleQueue ) then
					QJN_addQuickJoinNotification(playerName, " is queued for:", queueSummaryName);
				end
			end
		end
	elseif ( event == "PLAYER_LOGIN" ) then
		loadOptions();
		init_QJN_OptionsPanel();
		init_toastContainer();
	end
end

QJNframe:SetScript("OnEvent", QJNframe.OnEvent);

SLASH_QJN_QJNDEBUG1 = "/qjndebug";
SlashCmdList["QJN_QJNDEBUG"] = function (msg) 
	if ( msg == "add" ) then
		toastContainer:addToast("testName", "queueueueueued", "testQueueDescription");
	end
end