local function newSlider(parent, name, description, minRange, maxRange, stepSize, startValue, onValueChangedHook)
	--mostly from AdvancedInterfaceOptions
	local label = name
	local slider = CreateFrame('Slider', 'QJN_Options_Slider' .. name, parent, 'OptionsSliderTemplate')

	slider.name = name
	stepSize = stepSize or 1
	slider:SetValueStep(stepSize)
	slider:SetObeyStepOnDrag(true)

	slider:SetMinMaxValues(minRange, maxRange)
	slider.minText = _G[slider:GetName() .. 'Low']
	slider.maxText = _G[slider:GetName() .. 'High']
	slider.text = _G[slider:GetName() .. 'Text']
	slider.minText:SetText(minRange)
	slider.maxText:SetText(maxRange)
	slider.text:SetText(label)
	slider:SetValue(startValue);

	local valueText = slider:CreateFontString(nil, nil, 'GameFontHighlight')
	valueText:SetPoint('TOP', slider, 'BOTTOM', 0, -5)
	valueText:SetText(startValue);
	slider.valueText = valueText
	slider:HookScript('OnValueChanged', function(self, value)
		local factor = 1 / stepSize
		value = floor(value * factor + 0.5) / factor
		valueText:SetText(value)
	end)
	slider:HookScript("OnValueChanged", onValueChangedHook);

	slider.tooltipText = label
	slider.tooltipRequirement = description
	return slider
end

local function newCheckbox(parent, name, description, optionName)
	--mostly from AdvancedInterfaceOptions
	local label = name
	local check = CreateFrame("CheckButton", "QJN_Options_Checkbox" .. label, parent, "InterfaceOptionsCheckButtonTemplate")
	check.optionName = optionName

	check.GetValue = function(self)
		return QJN_Options[self.optionName]
	end
	check.SetValue = function(self, checked)
		QJN_Options[self.optionName] = checked
	end
	check:SetChecked(check:GetValue())
	check:SetScript('OnShow', function(self) self:SetChecked(self:GetValue()) end)
	check:SetScript("OnClick", function(self)
		local checked = self:GetChecked()
		PlaySound(checked and "igMainMenuOptionCheckBoxOn" or "igMainMenuOptionCheckBoxOff")
		self:SetValue(checked)
	end)
	check.label = _G[check:GetName() .. "Text"]
	check.label:SetText(label)
	check.tooltipText = label
	check.tooltipRequirement = description
	return check
end

function init_QJN_OptionsPanel () 
	local QJN_OptionsPanel = CreateFrame("FRAME");
	QJN_OptionsPanel.name = "QuickJoinNotifications"
	InterfaceOptions_AddCategory(QJN_OptionsPanel);

	--update this value after every element and use it to position the next one
	local nextAnchor = nil;
	
	local title = QJN_OptionsPanel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	title:SetPoint("TOPLEFT", 16, -16);
	title:SetText("QuickJoinNotifications");
	nextAnchor = title
	
	local QJN_Options_Chat = newCheckbox(QJN_OptionsPanel, "Chat Output", "Outputs new groups to the default chat.", "CHAT_ENABLED")
	QJN_Options_Chat:SetPoint("TOPLEFT", nextAnchor, "BOTTOMLEFT", 0, -15)
	nextAnchor = QJN_Options_Chat
	
	local QJN_Options_Toasts = newCheckbox(QJN_OptionsPanel, "Toast Output", "Outputs new groups in a little window above the chat.", "TOASTS_ENABLED")
	QJN_Options_Toasts:SetPoint("TOPLEFT", nextAnchor, "BOTTOMLEFT", 0, -10)
	nextAnchor = QJN_Options_Toasts

	local QJN_Options_Width = newSlider(QJN_OptionsPanel, "Width", "Set the width of a QuickJoin notification.", 100, 600, 10, QJN_Options["TOASTFRAME_WIDTH"], function (self, value)
		QJN_Options["TOASTFRAME_WIDTH"] = value
		QJN_applyOptions()
	end)
	QJN_Options_Width:ClearAllPoints();
	QJN_Options_Width:SetPoint("TOPLEFT", nextAnchor, "BOTTOMLEFT", 0, -15);
	nextAnchor = QJN_Options_Width
	
	local QJN_Options_Height = newSlider(QJN_OptionsPanel, "Height", "Set the height of a QuickJoin notification.", 50, 250, 5, QJN_Options["TOASTFRAME_HEIGHT"], function (self, value)
		QJN_Options["TOASTFRAME_HEIGHT"] = value
		QJN_applyOptions()
	end)
	QJN_Options_Height:ClearAllPoints();
	QJN_Options_Height:SetPoint("TOPLEFT", nextAnchor, "TOPRIGHT", 20, 0);

	
	--[[
	local QJN_Options_GrowdirectionLabel = QJN_OptionsPanel:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
	QJN_Options_GrowdirectionLabel:SetPoint('TOPLEFT', nextAnchor, 'BOTTOMLEFT', 0, -20)
	QJN_Options_GrowdirectionLabel:SetText('Select grow direction:')
	local QJN_Options_Growdirection = CreateFrame("Frame", "QJN_Options_Growdirection", QJN_OptionsPanel, "UIDropDownMenuTemplate")
	QJN_Options_Growdirection:SetPoint("TOPLEFT", QJN_Options_GrowdirectionLabel, "TOPRIGHT", 20, 0);
	QJN_Options_Growdirection.initialize = function(dropdown)
		local directions = { "TOP", "BOTTOM" }
		for k, v in pairs(directions) do
			local info = UIDropDownMenu_CreateInfo();
			info.text = v;
			info.value = v;
			info.func = function(self)
				QJN_Options["TOASTFRAME_GROWDIRECTION"] = self.value;
				QJN_applyOptions();
				UIDropDownMenu_SetSelectedValue(dropdown, self.value);
			end
			UIDropDownMenu_AddButton(info);
		end
		UIDropDownMenu_SetSelectedValue(dropdown, QJN_Options["TOASTFRAME_GROWDIRECTION"]);
	end
	QJN_Options_Growdirection:HookScript("OnShow", QJN_Options_Growdirection.initialize);
	nextAnchor = QJN_Options_GrowdirectionLabel
	--]]
	
	--###########TEST BUTTON SHOULD ALWAYS BE THE LAST ELEMENT###########
	local QJN_Options_AddTestToast = CreateFrame("Button", "QJN_Options_TestButton", QJN_OptionsPanel, "UIPanelButtonTemplate");
	QJN_Options_AddTestToast:SetPoint("TOPLEFT", nextAnchor, "BOTTOMLEFT", 0, -20)
	-- QJN_Options_AddTestToast:SetSize(40, 32) -- width, height
	QJN_Options_AddTestToast:SetText("Add a test notification")
	QJN_Options_AddTestToast:SetWidth(QJN_Options_AddTestToast:GetTextWidth() + 20);
	QJN_Options_AddTestToast:SetScript("OnClick", function()
		QJN_addQuickJoinNotification("Test Name", " is testing:", "Test description");
	end)
end

--QJN_Options
-- local QJN_Defaults["TOASTFRAME_WIDTH"] = 400;
-- local QJN_Defaults["TOASTFRAME_HEIGHT"] = 80;
-- local QJN_Defaults["TOASTFRAME_PADDING"] = 10;
-- local QJN_Defaults["TOASTFRAME_MARGIN"] = 15;
-- local QJN_Defaults["TOASTCONTAINER_YOFFSET"] = 40;
-- local QJN_Defaults["TOASTCONTAINER_MAXTOASTS"] = 3;
-- local QJN_Defaults["TOASTFRAME_GROWDIRECTION"] = "TOP";